/*
 * This file is part of the OdinMS Maple Story Server
 * Copyright (C) 2008 Patrick Huy <patrick.huy@frz.cc>
 * Matthias Butz <matze@odinms.de>
 * Jan Christian Meyer <vimes@odinms.de>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation version 3 as published by
 * the Free Software Foundation. You may not use, modify or distribute
 * this program under any other version of the GNU Affero General Public
 * License.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import constants.ServerConstants;
import tools.ExternalCodeTableGetter;
import tools.IntValueHolder;
import tools.logger.Logger;
import tools.logger.Logger.LogFile;
import tools.logger.Logger.LogType;

public enum SendOpcode implements IntValueHolder{
	BEGIN_SOCKET,
	CheckPasswordResult,
	GUEST_ID_LOGIN,
	ACCOUNT_INFO,
	CheckUserLimitResult,
	SetAccountResult,
	ConfirmEULAResult,
	CheckPinCodeResult,
	UpdatePinCodeResult,
	ViewAllCharResult,
	SelectCharacterByVACResult,
	WorldInformation,
	CHARLIST,
	SERVER_IP,
	CheckDuplicatedIDResult,
	CreateNewCharacterResult,
	DeleteCharacterResult,
	CHANGE_CHANNEL,
	AliveReq,
	KOREAN_INTERNET_CAFE_SHIT,
	CHANNEL_SELECTED,
	HACKSHIELD_REQUEST,
	RELOG_RESPONSE,
	CHECK_CRC_RESULT,
	LAST_CONNECTED_WORLD,
	RECOMMENDED_WORLD_MESSAGE,
	CHECK_SPW_RESULT,
	INVENTORY_OPERATION,
	INVENTORY_GROW,
	STAT_CHANGED,
	GIVE_BUFF,
	CANCEL_BUFF,
	FORCED_STAT_SET,
	FORCED_STAT_RESET,
	UPDATE_SKILLS,
	SKILL_USE_RESULT,
	FAME_RESPONSE,
	SHOW_STATUS_INFO,
	OPEN_FULL_CLIENT_DOWNLOAD_LINK,
	MEMO_RESULT,
	MAP_TRANSFER_RESULT,
	ANTI_MACRO_RESULT,
	CLAIM_RESULT,
	CLAIM_AVAILABLE_TIME,
	CLAIM_STATUS_CHANGED,
	SET_TAMING_MOB_INFO,
	QUEST_CLEAR,
	ENTRUSTED_SHOP_CHECK_RESULT,
	SKILL_LEARN_ITEM_RESULT,
	GATHER_ITEM_RESULT,
	SORT_ITEM_RESULT,
	SUE_CHARACTER_RESULT,
	TRADE_MONEY_LIMIT,
	SET_GENDER,
	GUILD_BBS_PACKET,
	CHAR_INFO,
	PARTY_OPERATION,
	BUDDYLIST,
	GUILD_OPERATION,
	ALLIANCE_OPERATION,
	SPAWN_PORTAL,
	SERVERMESSAGE,
	INCUBATOR_RESULT,
	SHOP_SCANNER_RESULT,
	SHOP_LINK_RESULT,
	MARRIAGE_REQUEST,
	MARRIAGE_RESULT,
	WEDDING_GIFT_RESULT,
	NOTIFY_MARRIED_PARTNER_MAP_TRANSFER,
	WEDDING_PHOTO,
	CASH_PET_FOOD_RESULT,
	SET_WEEK_EVENT_MESSAGE,
	SET_POTION_DISCOUNT_RATE,
	BRIDLE_MOB_CATCH_FAIL,
	IMITATED_NPC_RESULT,
	IMITATED_NPC_DATA,
	LIMITED_NPC_DISABLE_INFO,
	MONSTER_BOOK_SET_CARD,
	MONSTER_BOOK_SET_COVER,
	HOUR_CHANGED,
	MINIMAP_ON_OFF,
	CONSULT_AUTHKEY_UPDATE,
	CLASS_COMPETITION_AUTHKEY_UPDATE,
	WEB_BOARD_AUTHKEY_UPDATE,
	SESSION_VALUE,
	PARTY_VALUE,
	FIELD_SET_VARIABLE,
	BONUS_EXP_RATE_CHANGED,
	FAMILY_CHART_RESULT,
	FAMILY_INFO_RESULT,
	FAMILY_RESULT,
	FAMILY_JOIN_REQUEST,
	FAMILY_JOIN_REQUEST_RESULT,
	FAMILY_JOIN_ACCEPTED,
	FAMILY_PRIVILEGE_LIST,
	FAMILY_FAMOUS_POINT_INC_RESULT,
	FAMILY_NOTIFY_LOGIN_OR_LOGOUT,
	FAMILY_SET_PRIVILEGE,
	FAMILY_SUMMON_REQUEST,
	NOTIFY_LEVELUP,
	NOTIFY_MARRIAGE,
	NOTIFY_JOB_CHANGE,
	MAPLE_TV_USE_RES,
	AVATAR_MEGAPHONE_RESULT,
	SET_AVATAR_MEGAPHONE,
	CLEAR_AVATAR_MEGAPHONE,
	CANCEL_NAME_CHANGE_RESULT,
	CANCEL_TRANSFER_WORLD_RESULT,
	DESTROY_SHOP_RESULT,
	FAKE_GM_NOTICE,
	SUCCESS_IN_USE_GACHAPON_BOX,
	NEW_YEAR_CARD_RES,
	RANDOM_MORPH_RES,
	CANCEL_NAME_CHANGE_BY_OTHER,
	SET_BUY_EQUIP_EXT,
	SET_PASSENGER_REQUEST,
	SCRIPT_PROGRESS_MESSAGE,
	DATA_CRC_CHECK_FAILED,
	MACRO_SYS_DATA_INIT,
	BEGIN_STAGE,
	SetField,
	SetITC,
	SetCashShop,
	END_STAGE,
	SET_BACK_EFFECT,
	SET_MAP_OBJECT_VISIBLE,
	CLEAR_BACK_EFFECT,
	BLOCKED_MAP,
	BLOCKED_SERVER,
	FORCED_MAP_EQUIP,
	MULTICHAT,
	WHISPER,
	SPOUSE_CHAT,
	SUMMON_ITEM_INAVAILABLE,
	FIELD_EFFECT,
	FIELD_OBSTACLE_ONOFF,
	FIELD_OBSTACLE_ONOFF_STATUS,
	FIELD_OBSTACLE_ALL_RESET,
	BLOW_WEATHER,
	PLAY_JUKEBOX,
	ADMIN_RESULT,
	OX_QUIZ,
	GMEVENT_INSTRUCTIONS,
	CLOCK,
	CONTI_MOVE,
	CONTI_STATE,
	SET_QUEST_CLEAR,
	SET_QUEST_TIME,
	WARN_MESSAGE,
	SET_OBJECT_STATE,
	STOP_CLOCK,
	ARIANT_ARENA_SHOW_RESULT,
	PYRAMID_GAUGE,
	PYRAMID_SCORE,
	QUICKSLOT_SET,
	SPAWN_PLAYER,
	REMOVE_PLAYER_FROM_MAP,
	CHATTEXT,
	CHATTEXT1,
	CHALKBOARD,
	UPDATE_CHAR_BOX,
	SHOW_CONSUME_EFFECT,
	SHOW_SCROLL_EFFECT,
	SHOW_ITEM_HYPER_UPGRADE_EFFECT,
	SHOW_ITEM_OPTION_UPGRADE_EFFECT,
	SHOW_ITEM_RELEASE_EFFECT,
	SHOW_ITEM_UNRELEASE_EFFECT,
	FOLLOW_CHARACTER,
	SPAWN_PET,
	EVOLVE_PET,
	MOVE_PET,
	PET_CHAT,
	PET_NAMECHANGE,
	PET_EXCEPTION_LIST_RESULT,
	PET_COMMAND,
	SPAWN_SPECIAL_MAPOBJECT,
	REMOVE_SPECIAL_MAPOBJECT,
	MOVE_SUMMON,
	SUMMON_ATTACK,
	DAMAGE_SUMMON,
	SUMMON_SKILL,
	SPAWN_DRAGON,
	MOVE_DRAGON,
	REMOVE_DRAGON,
	MOVE_PLAYER,
	CLOSE_RANGE_ATTACK,
	RANGED_ATTACK,
	MAGIC_ATTACK,
	ENERGY_ATTACK,
	SKILL_EFFECT,
	CANCEL_SKILL_EFFECT,
	DAMAGE_PLAYER,
	FACIAL_EXPRESSION,
	SHOW_ITEM_EFFECT,
	SHOW_CHAIR,
	UPDATE_CHAR_LOOK,
	SHOW_FOREIGN_EFFECT,
	GIVE_FOREIGN_BUFF,
	CANCEL_FOREIGN_BUFF,
	UPDATE_PARTYMEMBER_HP,
	GUILD_NAME_CHANGED,
	GUILD_MARK_CHANGED,
	CANCEL_CHAIR,
	USER_EFFECT_LOCAL,
	DOJO_WARP_UP,
	LUCKSACK_PASS,
	LUCKSACK_FAIL,
	MESO_BAG_MESSAGE,
	UPDATE_QUEST_INFO,
	PLAYER_HINT,
	MAKER_RESULT,
	KOREAN_EVENT,
	OPEN_UI,
	LOCK_UI,
	DISABLE_UI,
	SPAWN_GUIDE,
	TALK_GUIDE,
	SHOW_COMBO,
	NOTICE_MSG,
	CHAT_MSG,
	FOLLOW_CHARACTER_FAILED,
	COOLDOWN,
	SPAWN_MONSTER,
	KILL_MONSTER,
	SPAWN_MONSTER_CONTROL,
	MOVE_MONSTER,
	MOVE_MONSTER_RESPONSE,
	MobStatSet,
	MobStatReset,
	RESET_MONSTER_ANIMATION,
	DAMAGE_MONSTER,
	MONSTER_SPECIAL_EFFECT_BY_SKILL,
	MOB_CRC_KEY_CHANGED,
	SHOW_MONSTER_HP,
	SHOW_DRAGGED,
	CATCH_MONSTER,
	SHOW_MAGNET,
	SPAWN_NPC,
	REMOVE_NPC,
	SPAWN_NPC_REQUEST_CONTROLLER,
	NPC_ACTION,
	NPC_SPECIAL_ACTION,
	SET_NPC_SCRIPTABLE,
	SPAWN_HIRED_MERCHANT,
	DESTROY_HIRED_MERCHANT,
	UPDATE_HIRED_MERCHANT,
	DROP_ITEM_FROM_MAPOBJECT,
	REMOVE_ITEM_FROM_MAP,
	SPAWN_KITE_MESSAGE,
	SPAWN_KITE,
	DESTROY_KITE,
	SPAWN_MIST,
	REMOVE_MIST,
	SPAWN_DOOR,
	REMOVE_DOOR,
	REACTOR_HIT,
	REACTOR_SPAWN,
	REACTOR_DESTROY,
	REACTOR_MOVE,
	SNOWBALL_STATE,
	HIT_SNOWBALL,
	SNOWBALL_MESSAGE,
	LEFT_KNOCK_BACK,
	COCONUT_HIT,
	COCONUT_SCORE,
	GUILD_BOSS_HEALER_MOVE,
	GUILD_BOSS_PULLEY_STATE_CHANGE,
	MONSTER_CARNIVAL_START,
	MONSTER_CARNIVAL_OBTAINED_CP,
	MONSTER_CARNIVAL_PARTY_CP,
	MONSTER_CARNIVAL_SUMMON,
	MONSTER_CARNIVAL_MESSAGE,
	MONSTER_CARNIVAL_DIED,
	MONSTER_CARNIVAL_LEAVE,
	MONSTER_CARNIVAL_RESULT,
	ARIANT_ARENA_USER_SCORE,
	SHEEP_RANCH_INFO,
	SHEEP_RANCH_CLOTHES,
	ARIANT_SCORE,
	HORNTAIL_CAVE,
	ZAKUM_SHRINE,
	NPC_TALK,
	OPEN_NPC_SHOP,
	CONFIRM_SHOP_TRANSACTION,
	ADMIN_SHOP_RESULT,
	ADMIN_SHOP_COMMODITY,
	STORAGE,
	FREDRICK_MESSAGE,
	FREDRICK,
	RPS_GAME,
	MESSENGER,
	PLAYER_INTERACTION,
	TOURNAMENT,
	TOURNAMENT_MATCH_TABLE,
	TOURNAMENT_SET_PRIZE,
	TOURNAMENT_UEW,
	TOURNAMENT_CHARACTERS,
	WEDDING_PROGRESS,
	WEDDING_CEREMONY_END,
	PARCEL,
	CHARGE_PARAM_RESULT,
	QUERY_CASH_RESULT,
	CASHSHOP_OPERATION,
	KEYMAP,
	AUTO_HP_POT,
	AUTO_MP_POT,
	SEND_TV,
	REMOVE_TV,
	ENABLE_TV,
	MTS_OPERATION2,
	MTS_OPERATION,
	VICIOUS_HAMMER,
	GACHAPON_STAMP_ITEM_RESULT,
	CASH_ITEM_GACHAPON_RESULT,
	CASH_GACHAPON_OPEN_RESULT,
	ACCOUNT_MORE_INFO,
	FIND_FRIEND,
	VEGA_RESULT,
	VEGA_FAIL,;

	private int code = -2;

	@Override
	public void setValue(int code){
		this.code = code;
	}

	@Override
	public int getValue(){
		return code;
	}

	public static SendOpcode getOpcodeByOp(int op){
		for(SendOpcode opcode : values()){
			if(opcode.getValue() == op) return opcode;
		}
		return null;
	}

	public static Properties getDefaultProperties() throws FileNotFoundException, IOException{
		Properties props = new Properties();
		try(FileInputStream fis = new FileInputStream(new File("sendops-" + ServerConstants.VERSION + ".properties"))){
			props.load(fis);
		}
		return props;
	}

	static{
		try{
			ExternalCodeTableGetter.populateValues(getDefaultProperties(), values());
		}catch(IOException e){
			Logger.log(LogType.ERROR, LogFile.EXCEPTION, e);
		}
	}
}
