/*
 * This file is part of the OdinMS Maple Story Server
 * Copyright (C) 2008 Patrick Huy <patrick.huy@frz.cc>
 * Matthias Butz <matze@odinms.de>
 * Jan Christian Meyer <vimes@odinms.de>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation version 3 as published by
 * the Free Software Foundation. You may not use, modify or distribute
 * this program under any other version of the GNU Affero General Public
 * License.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net.server;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

import client.MapleCharacter;
import client.MapleClient;
import constants.ServerConstants;
import net.MTSAuction;
import net.server.channel.Channel;
import net.server.world.World;
import server.MapleWedding;
import server.TimerManager;
import tools.AutoJCE;
import tools.DatabaseConnection;
import tools.Pair;
import tools.Triple;
import tools.logger.Logger;
import tools.logger.Logger.LogFile;
import tools.logger.Logger.LogType;

public class Server implements Runnable{

	private List<Map<Integer, String>> channels = new LinkedList<>();
	private List<World> worlds = new ArrayList<>();
	private Properties subnetInfo = new Properties();
	private static Server instance = null;
	private List<Pair<Integer, String>> worldRecommendedList = new LinkedList<>();
	private boolean online = false;
	public static long uptime = System.currentTimeMillis();
	private List<MapleWedding> weddings = new LinkedList<>();
	public static int highestWeddingID = 0;
	public boolean shuttingDown = false;

	@Deprecated
	public static Server getInstance(){
		/*if(instance == null){
			instance = new Server();
		}*/
		return null;
	}

	public boolean isOnline(){
		return online;
	}

	public List<Pair<Integer, String>> worldRecommendedList(){
		return worldRecommendedList;
	}

	public void removeChannel(int worldid, int channel){
		channels.remove(channel);
		World world = worlds.get(worldid);
		if(world != null){
			world.removeChannel(channel);
		}
	}

	public Channel getChannel(int world, int channel){
		return worlds.get(world).getChannel(channel);
	}

	public MapleClient getClientGlobal(int id){
		for(World world : getWorlds()){
			for(MapleCharacter player : world.getAllCharacters()){
				if(player.getAccountID() == id) return player.getClient();
			}
		}
		return null;
	}

	public MapleClient getClientGlobal(String id){
		for(World world : getWorlds()){
			for(MapleCharacter player : world.getAllCharacters()){
				if(player.getClient().getAccountName().equalsIgnoreCase(id)) return player.getClient();
			}
		}
		return null;
	}

	public List<Channel> getChannelsFromWorld(int world){
		return worlds.get(world).getChannels();
	}

	public List<Channel> getAllChannels(){
		List<Channel> channelz = new ArrayList<>();
		for(World world : worlds){
			for(Channel ch : world.getChannels()){
				channelz.add(ch);
			}
		}
		return channelz;
	}

	public String getIP(int world, int channel){
		return channels.get(world).get(channel);
	}

	@Override
	public void run(){
		System.setProperty("wzpath", "wz");
		System.out.println("Vertisy v" + ServerConstants.VERSION + " starting up.");
		AutoJCE.removeCryptographyRestrictions();
		/*if(ServerConstants.SHUTDOWNHOOK){
			Runtime.getRuntime().addShutdownHook(new Thread(shutdown(false)));
		}*/
		try(Connection c = DatabaseConnection.getConnection()){
			PreparedStatement ps = c.prepareStatement("UPDATE accounts SET loggedin = 0");
			ps.executeUpdate();
			ps.close();
			ps = c.prepareStatement("UPDATE characters SET HasMerchant = 0");
			ps.executeUpdate();
			ps.close();
		}catch(SQLException sqle){
			Logger.log(LogType.ERROR, LogFile.EXCEPTION, sqle);
		}
		// tMan.register(tMan.purge(), 300000);//Purging ftw...
		TimerManager.getInstance().register("rankingWorker", new RankingWorker(), ServerConstants.RANKING_INTERVAL);
		TimerManager.getInstance().register("MTSAuction", new MTSAuction(), ServerConstants.MTS_AUCTION_INTERVAL);
		try(var con = DatabaseConnection.getConnection();
			ResultSet rs = con.createStatement().executeQuery("SELECT id FROM weddings where id > 0")){
			while(rs.next()){
				MapleWedding wedding = new MapleWedding(rs.getInt("id"));
				wedding.loadFromDB();
				addWedding(wedding);
			}
		}catch(Exception ex){
			Logger.log(LogType.ERROR, LogFile.EXCEPTION, ex);
		}
		System.out.println("Vertisy is now online.");
		online = true;
	}

	public void shutdown(){
		System.out.println("Server offline.");
		System.exit(0);// BOEIEND :D
	}

	public Properties getSubnetInfo(){
		return subnetInfo;
	}

	public Set<Integer> getChannelServer(int world){
		return new HashSet<>(channels.get(world).keySet());
	}

	public byte getHighestChannelId(){
		byte highest = 0;
		for(Iterator<Integer> it = channels.get(0).keySet().iterator(); it.hasNext();){
			Integer channel = it.next();
			if(channel != null && channel.intValue() > highest){
				highest = channel.byteValue();
			}
		}
		return highest;
	}

	public void broadcastMessage(final byte[] packet){
		for(World world : getWorlds()){
			world.broadcastPacket(packet);
		}
	}

	public void broadcastGMMessage(final byte[] packet){
		for(World world : getWorlds()){
			world.broadcastGMPacket(packet);
		}
	}

	public boolean isGmOnline(){
		for(World world : getWorlds()){
			for(Channel ch : world.getChannels()){
				for(MapleCharacter player : ch.getPlayerStorage().getAllCharacters()){
					if(player.isGM()) return true;
				}
			}
		}
		return false;
	}

	public World getWorld(int id){
		return worlds.get(id);
	}

	public List<World> getWorlds(){
		return worlds;
	}

	public MapleWedding getWeddingByID(int marriageID){
		for(int i = 0; i < weddings.size(); i++){
			MapleWedding wedding = weddings.get(i);
			if(wedding.getMarriageID() == marriageID){
				wedding.index = i;
				return wedding;
			}
		}
		return null;
	}

	public MapleWedding getWedding(int player){
		return getWedding(player, 0);
	}

	public MapleWedding getWedding(int player, int requiredStatus){
		for(int i = 0; i < weddings.size(); i++){
			MapleWedding wedding = weddings.get(i);
			if(wedding.getStatus() >= requiredStatus){
				if(wedding.getPlayer1() == player || wedding.getPlayer2() == player){
					wedding.index = i;
					return wedding;
				}
			}
		}
		return null;
	}

	public void addWedding(MapleWedding wedding){
		if(wedding.index == -1){
			weddings.add(wedding);
		}else{
			weddings.set(wedding.index, wedding);
		}
	}
}
