/*
 * This file is part of the OdinMS Maple Story Server
 * Copyright (C) 2008 Patrick Huy <patrick.huy@frz.cc>
 * Matthias Butz <matze@odinms.de>
 * Jan Christian Meyer <vimes@odinms.de>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation version 3 as published by
 * the Free Software Foundation. You may not use, modify or distribute
 * this program under any other version of the GNU Affero General Public
 * License.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net.server.channel.handlers;

import client.MapleClient;
import client.SkillMacro;
import client.autoban.AutobanFactory;
import constants.GameConstants;
import net.AbstractMaplePacketHandler;
import tools.data.input.SeekableLittleEndianAccessor;

public final class SkillMacroHandler extends AbstractMaplePacketHandler{

	@Override
	public final void handlePacket(SeekableLittleEndianAccessor slea, MapleClient c){
		int num = slea.readByte();
		for(int i = 0; i < num; i++){
			String name = slea.readMapleAsciiString();
			int shout = slea.readByte();
			int skill1 = slea.readInt();
			int skill2 = slea.readInt();
			int skill3 = slea.readInt();
			if(skill1 != 0 && c.getPlayer().getSkillLevel(skill1) == 0){
				AutobanFactory.PACKET_EDIT.autoban(c.getPlayer(), "Tried to set a macro with skill: " + skill1 + " without any levels in that skill");
			}
			if(skill2 != 0 && c.getPlayer().getSkillLevel(skill2) == 0){
				AutobanFactory.PACKET_EDIT.autoban(c.getPlayer(), "Tried to set a macro with skill: " + skill2 + " without any levels in that skill");
			}
			if(skill3 != 0 && c.getPlayer().getSkillLevel(skill3) == 0){
				AutobanFactory.PACKET_EDIT.autoban(c.getPlayer(), "Tried to set a macro with skill: " + skill3 + " without any levels in that skill");
			}
			if(GameConstants.isHiddenSkills(skill1)){
				AutobanFactory.PACKET_EDIT.autoban(c.getPlayer(), "Tried to set a macro with skill: " + skill1 + " when its a hidden skill.");
			}
			if(GameConstants.isHiddenSkills(skill2)){
				AutobanFactory.PACKET_EDIT.autoban(c.getPlayer(), "Tried to set a macro with skill: " + skill2 + " when its a hidden skill.");
			}
			if(GameConstants.isHiddenSkills(skill3)){
				AutobanFactory.PACKET_EDIT.autoban(c.getPlayer(), "Tried to set a macro with skill: " + skill3 + " when its a hidden skill.");
			}
			SkillMacro macro = new SkillMacro(skill1, skill2, skill3, name, shout, i);
			c.getPlayer().updateMacros(i, macro);
		}
	}
}
