/*
 * This file is part of the OdinMS Maple Story Server
 * Copyright (C) 2008 Patrick Huy <patrick.huy@frz.cc>
 * Matthias Butz <matze@odinms.de>
 * Jan Christian Meyer <vimes@odinms.de>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation version 3 as published by
 * the Free Software Foundation. You may not use, modify or distribute
 * this program under any other version of the GNU Affero General Public
 * License.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package net.server.channel.handlers;

import java.util.ArrayList;
import java.util.List;

import client.MapleCharacter;
import client.MapleClient;
import client.Skill;
import client.SkillFactory;
import net.AbstractMaplePacketHandler;
import server.MapleStatEffect;
import server.life.MapleMonster;
import server.maps.objects.MapleSummon;
import tools.data.input.SeekableLittleEndianAccessor;
import tools.packets.field.SummonedPool;

public final class SummonDamageHandler extends AbstractMaplePacketHandler{

	public final class SummonAttackEntry{

		private int monsterOid;
		private int damage;

		public SummonAttackEntry(int monsterOid, int damage){
			this.monsterOid = monsterOid;
			this.damage = damage;
		}

		public int getMonsterOid(){
			return monsterOid;
		}

		public int getDamage(){
			return damage;
		}
	}

	@Override
	public void handlePacket(SeekableLittleEndianAccessor slea, MapleClient c){
		int oid = slea.readInt();
		MapleCharacter player = c.getPlayer();
		if(!player.isAlive()) return;
		MapleSummon summon = null;
		for(MapleSummon sum : player.getSummons().values()){
			if(sum.getObjectId() == oid){
				summon = sum;
			}
		}
		if(summon == null) return;
		Skill summonSkill = SkillFactory.getSkill(summon.getSkill());
		MapleStatEffect summonEffect = summonSkill.getEffect(summon.getSkillLevel());
		slea.skip(8);
		slea.readInt();
		slea.skip(8);
		List<SummonAttackEntry> allDamage = new ArrayList<>();
		byte direction = slea.readByte();
		slea.skip(8);
		int numAttacked = slea.readByte();
		slea.skip(8);// Thanks Gerald :D, I failed lol (mob x,y and summon x,y)
		for(int x = 0; x < numAttacked; x++){
			int monsterOid = slea.readInt(); // attacked oid
			slea.skip(18);
			int damage = slea.readInt();
			// System.out.println(damage);
			allDamage.add(new SummonAttackEntry(monsterOid, damage));
		}
		player.getMap().broadcastMessage(player, SummonedPool.summonAttack(player.getId(), summon.getSkill(), direction, allDamage), summon.getPosition());
		for(SummonAttackEntry attackEntry : allDamage){
			int damage = attackEntry.getDamage();
			MapleMonster target = player.getMap().getMonsterByOid(attackEntry.getMonsterOid());
			if(target != null){
				if(damage > 0 && summonEffect.getMonsterStati().size() > 0){
					if(summonEffect.makeChanceResult()){
						target.applyStatus(player, summonEffect.getMonsterStati(), summonSkill, summonEffect.isPoison(), false);
					}
				}
				player.getMap().damageMonster(player, target, damage);
			}
		}
	}
}