/*
 * This file is part of the OdinMS Maple Story Server
 * Copyright (C) 2008 Patrick Huy <patrick.huy@frz.cc>
 * Matthias Butz <matze@odinms.de>
 * Jan Christian Meyer <vimes@odinms.de>
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation version 3 as published by
 * the Free Software Foundation. You may not use, modify or distribute
 * this program under any other version of the GNU Affero General Public
 * License.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package scripting.portal;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import client.MapleCharacter;
import client.MapleClient;
import scripting.AbstractPlayerInteraction;
import server.MaplePortal;
import tools.DatabaseConnection;
import tools.logger.Logger;
import tools.logger.Logger.LogFile;
import tools.logger.Logger.LogType;
import tools.packets.UserLocal;

public class PortalPlayerInteraction extends AbstractPlayerInteraction{

	private MaplePortal portal;

	public PortalPlayerInteraction(MapleClient c, MaplePortal portal){
		super(c);
		this.portal = portal;
	}

	public MaplePortal getPortal(){
		return portal;
	}

	public boolean hasLevel30Character(){
		PreparedStatement ps = null;
		ResultSet rs = null;
		try(var con = DatabaseConnection.getConnection()){
			ps = con.prepareStatement("SELECT `level` FROM `characters` WHERE accountid = ?");
			ps.setInt(1, getPlayer().getAccountID());
			rs = ps.executeQuery();
			while(rs.next()){
				if(rs.getInt("level") >= 30){
					ps.close();
					rs.close();
					return true;
				}
			}
		}catch(SQLException sqle){
			Logger.log(LogType.ERROR, LogFile.EXCEPTION, sqle);
		}finally{
			try{
				if(ps != null && !ps.isClosed()){
					ps.close();
				}
				if(rs != null && !rs.isClosed()){
					rs.close();
				}
			}catch(SQLException ex){
				Logger.log(LogType.ERROR, LogFile.EXCEPTION, ex);
			}
		}
		return false;
	}

	public void blockPortal(){
		c.getPlayer().blockPortal(getPortal().getScriptName());
	}

	public void unblockPortal(){
		c.getPlayer().unblockPortal(getPortal().getScriptName());
	}

	public void playPortalSound(){
		c.announce(UserLocal.UserEffect.playPortalSound());
	}

	@Override
	public void gainExp(MapleCharacter player, int exp, double multiplier, boolean show, String logData){
		logData += "From Portal: " + portal.getId() + " Script: " + portal.getScriptName();
		super.gainExp(player, exp, multiplier, show, logData);
	}
}