package tools;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by MechAviv on 24/10/2019.
 */
public class HebrewUtil {
    // E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA EB EC ED EE EF F0 F1 F2 F3 F4 F5 F6 F7 F8 F9 FA = Client
    // D0 D1 D2 D3 D4 D5 D6 D7 D8 D9 DA DB DC DD DE DF E0 E1 E2 E3 E4 E5 E6 E7 E8 E9 EA = Server
    private static int CLIENT_START = 0xE0, CLIENT_END = 0xFA;
    private static int SERVER_START = 0xD0, SERVER_END = 0xEA;

    public static Map<Integer, Character> characterMap = new HashMap<>();

    static {
        characterMap.put(0xD0, 'א');
        characterMap.put(0xD1, 'ב');
        characterMap.put(0xD2, 'ג');
        characterMap.put(0xD3, 'ד');
        characterMap.put(0xD4, 'ה');
        characterMap.put(0xD5, 'ו');
        characterMap.put(0xD6, 'ז');
        characterMap.put(0xD7, 'ח');
        characterMap.put(0xD8, 'ט');
        characterMap.put(0xD9, 'י');
        characterMap.put(0xDA, 'ך');
        characterMap.put(0xDB, 'כ');
        characterMap.put(0xDC, 'ל');
        characterMap.put(0xDD, 'ם');
        characterMap.put(0xDE, 'מ');
        characterMap.put(0xDF, 'ן');
        characterMap.put(0xE0, 'נ');
        characterMap.put(0xE1, 'ס');
        characterMap.put(0xE2, 'ע');
        characterMap.put(0xE3, 'ף');
        characterMap.put(0xE4, 'פ');
        characterMap.put(0xE5, 'ץ');
        characterMap.put(0xE6, 'צ');
        characterMap.put(0xE7, 'ק');
        characterMap.put(0xE8, 'ר');
        characterMap.put(0xE9, 'ש');
        characterMap.put(0xEA, 'ת');
    }

    /**
     * Convert decoded string to server string
     *
     * @param text
     * @return
     */
    public static String decode(String text) {
        // Must be in server byte range
        String decoded = decodeFromClient(text);

        char[] charText = new char[decoded.length()];
        for (int i = 0; i < charText.length; i++) {
            charText[i] = characterMap.getOrDefault(decoded.charAt(i) & 0xFF, decoded.charAt(i));
        }
        return String.valueOf(charText);
    }

    /**
     * Convert from server to client string.
     *
     * @param text to encode.
     * @return client string.
     */
    public static String encode(String text) {
        // Encode part
        char[] charText = new char[text.length()];
        for (int i = 0; i < charText.length; i++) {
            int c = (text.charAt(i) & 0xFF);
            if (isServerHebrew(c)) {
                charText[i] = (char) (c + 0x10);
            } else {
                charText[i] = text.charAt(i);
            }
        }
        return String.valueOf(charText);
    }

    /**
     * Convert from client to sever string.
     *
     * @param text to decode.
     * @return server string.
     */
    private static String decodeFromClient(String text) {
        char[] charText = new char[text.length()];
        for (int i = 0; i < charText.length; i++) {
            int c = (text.charAt(i) & 0xFF);
            if (isClientHebrew(c)) {
                charText[i] = (char) (c - 0x10);
            } else {
                charText[i] = text.charAt(i);
            }
        }
        return String.valueOf(charText);
    }

    /**
     * Checks if the character in range of client hebrew bytes
     *
     * @param c char to check
     * @return if it hebrew
     */
    public static boolean isClientHebrew(int c) {
        return c >= CLIENT_START && c <= CLIENT_END;
    }

    /**
     * Checks if the character in range of server hebrew bytes
     *
     * @param c char to check
     * @return if it hebrew
     */
    public static boolean isServerHebrew(int c) {
        return c >= SERVER_START && c <= SERVER_END;
    }

    private static boolean isHebrewWord(String word) {
        for (int i = 0; i < word.length(); i++) {
            if (isServerHebrew(word.charAt(i) & 0xFF)) {
                return true;
            }
        }
        return false;
    }
}
